package WWW::Search::Easynews;

require 5.005;
use strict;
use vars qw/@ISA $VERSION $MAINTAINER/;
use Data::Dumper;

local $^W = 1;

use Carp qw(carp croak);
use WWW::Search qw(generic_option);
require WWW::SearchResult;

use HTML::TreeBuilder;
use LWP::UserAgent;
use HTTP::Request::Common qw/GET/;
use URI::Escape;

@ISA = qw(WWW::Search);
$VERSION = '0.12';
$MAINTAINER = q{Trond Michelsen <trondmm-perl@crusaders.no>};

my $result_count = 0;
my $ua = new LWP::UserAgent;

# Preloaded methods go here.
{
    my $native_query; # not pretty.. but it needs to be shared between these two functions.

    sub native_setup_search {
	my $self = shift;
	$native_query = shift;
	my $native_options_ref = shift;

	$self->user_agent("Mozilla/4.0 (compatible; MSIE 4.01; Windows 98)");
	$self->{_next_to_retrieve} = 0;

	my $groups = delete $native_options_ref->{groups};
	my $group;

	if (ref $groups) {
	    die qq["groups" must be a string or an arrayref] unless ref $groups eq "ARRAY";
	    $groups = [ @$groups ]; # don't want to delete the caller's original array
	    $group = shift @$groups;
	    $self->{_groups} = $groups;
	} else {
	    $group = $groups;
	    $self->{_groups} = [];
	}

	if (!defined($self->{_options})) {
	    $self->{_options} = {
		'search_url'  => 'http://members.easynews.com/search.html',
		'current_url' => 'http://members.easynews.com/current.html',
		'group'       => $group,
	    };
	};
	my($options_ref) = $self->{_options};
	if (defined($native_options_ref)) {
	    # Copy in new options.
	    foreach (keys %$native_options_ref) {
		$options_ref->{$_} = $native_options_ref->{$_};
	    };
	};
	$options_ref->{pageby} ||= $self->maximum_to_retrieve || 20;
	$options_ref->{pwfile} ||= "$ENV{HOME}/.easynews/login";
	$options_ref->{current}  = 0 unless exists $options_ref->{current};

	# We can't do a search unless we know what group to search in
	croak "Missing group\n" unless $options_ref->{group};
	
	# A username and password is essential
	croak "Missing username for easynews" unless defined $self->{_user};
	croak "Missing password for easynews" unless defined $self->{_pass};

	# We don't want to include the pwfilename in the search.
	delete $options_ref->{pwfile};
	$self->_build_base;	
    }

    sub _build_base {
	my $self = shift;
	my $options_ref = $self->{_options};

	# Process the options.
	# (Now in sorted order for consistency regarless of hash ordering.)

	my $options = join "&" => 
	    map  {join "=", $_, $options_ref->{$_}} 
            grep {! generic_option($_) } sort keys %$options_ref;
	
	$self->{_debug} = $options_ref->{'search_debug'};
	$self->{_debug} = 2 if ($options_ref->{'search_parse_debug'});
	$self->{_debug} = 0 if (!defined($self->{_debug}));
	
	# Finally figure out the url.
	$self->{_base_url} = 
	    $self->{_next_url} =
		( $self->{_options}{current} 
		  ? $self->{_options}{current_url}
		  : $self->{_options}{'search_url'} ) 
		    . qq(?$options&search=$native_query);
	print STDERR "Base URL: $self->{_base_url}\n" if $self->{_debug};
	return $self->{_base_url};
    }
}

sub native_retrieve_some {
    my ($self) = (@_);

    # fast exit if already done
    unless (defined $self->{_next_url}) {
	if ($self->_next_group) {return &native_retrieve_some} else {return undef}
    }

    # setup the cache for seen files;
    unless (exists $self->{_seen}) {
	$self->{_seen} = {};
    }

    # get some
    print STDERR __PACKAGE__ . "::native_retrieve_some: fetching " . $self->{_next_url} . "\n" if ($self->{_debug});
    my($res, $tree) = $self->_get_and_parse($self->{_next_url});
    $self->{response} = $res;
    unless ($res->is_success) {
	print STDERR $res->status_line if $self->{_debug};
	if ($self->_next_group) {return &native_retrieve_some} else {return undef}
    }

    my $files = $self->_get_files($tree);

    unless (defined $files) {   # no hits at all
	if ($self->_next_group) {return &native_retrieve_some} else {return undef}
    }
    if (@$files == 0) {  # should not happen. suggests we got hits, but couldn't parse any
	if ($self->_next_group) {return &native_retrieve_some} else {return undef}
    }
    my $hits_found = @$files;

    while (my $file = shift @$files) {
	if ( $self->{_seen}->{ $file->{unique} }++ ) {
	    $hits_found--;
	    next;
	}

	my $hit = new WWW::SearchResult;
	$hit->add_url($file->{url});
	$hit->title(exists $file->{realname} ? $file->{realname} : $file->{name});
	$hit->description($file->{subject});
	$hit->change_date($file->{posted});
	$hit->size($file->{bytesize});
	$hit->raw($file);

	my $orig = URI::Escape::uri_escape($hit->title);
	my $url = $file->{url};
	$url =~ s{(.*)/.*}{$1/$orig};
	$hit->add_url($url);

	push(@{$self->{cache}}, $hit);
    }
    return $hits_found;
}

#
# This function is called whenever native_retrieve_some() has retrieved all results from a query
# It checks if the user supplied more groups, and sets up a new query if he did.
# native_retrive_som re-calls itself if this function returns true (the next query)
#
sub _next_group {
    my $self = shift;
    if (@{ $self->{_groups} }) {
	$self->{_options}{group} = shift @{ $self->{_groups} };
	$result_count = 0;
	return $self->_build_base;
    } else {
	return undef;
    }
}

# internal
{ 
    my %time = (min   => 60,
		hours => 60 * 60,
		days  => 60 * 60 * 24,
		);
    my %known_size = ( "4.9 Megs" =>  5_000_000,
		       "7.8 Megs" =>  8_000_000,
		       "9.8 Megs" => 10_000_000,
		      "10.3 Megs" => 10_500_000,
		      "12.2 Megs" => 12_500_000,
		      "14.6 Megs" => 15_000_000,
		      "15.0 Megs" => 15_360_000,
		      "19.5 Megs" => 20_000_000,
		      );
    my %size = (bytes => 1,
		K     => 1024,
		Megs  => 1000 * 1024, # <- at Easynews... (*I* know better ;)
		);

    sub _get_files {
	my $self = shift;
	my ($tree) = @_;
	my @files;

	print STDERR "finding files..." if $self->{_debug};
	my $form = $tree->look_down(_tag => "form", name => "zip");
	return undef unless defined $form;

	my @results = $form->look_down(_tag => "table");
	shift @results;
	print STDERR " (done)\n" if $self->{_debug};

	my $center  = $tree->look_down(_tag => "center");
	unless ($result_count) {
	    $self->approximate_result_count($result_count = 
					    ($self->_get_result_count($center->as_text) || scalar @results)
					    + $self->approximate_result_count);
	}

	print STDERR "Extracting info" if $self->{_debug};
	for my $i (0..$#results) {
#	    print $results[$i]->as_HTML;
	    my @row  = ($results[$i]->look_down(_tag => "tr"))[1,2];

	    my $link = $row[0]->look_down(_tag => "a");
	    my $href = $link->attr("href");
	    if ($href =~ /^javascript:warnexe\(\'(.*)\'\)$/) {
		$href = $1;
	    }
	    my $url = URI->new_abs($href, $self->{_next_url})->as_string;
	    my ($checksums, $filename) = (split "/" => $url)[-2, -1];
	    my ($md5, $crc, $ext) = $checksums =~ /([0-9a-f]{32})0([0-9a-f]{8})\.(.*)/;

	    my ($age, $size, $poster, $hlink) = $row[1]->look_down(_tag => "td");
	    my $hurl = $hlink->look_down(_tag => "a")->attr("href");
	    my ($msgid) = $hurl =~ /messageid=(.*)/;
	    $msgid = uri_unescape($msgid);
	    $msgid =~ s/\s*//g;
	    my %file = (name      => $filename,
			url       => $url,
			md5       => $md5, 
			crc32     => $crc,
			unique    => $checksums,
			age       => $age->content->[0],
			size      => $size->content->[0],
			subject   => $link->content->[0],
			poster    => $poster->as_text,
			messageid => $msgid, 
			group     => $self->{_options}{group},
			);

#	    print Dumper \%file;
	    { # Make an attempt to determine the age of the file
		my ($value, $multiplier) = $file{age} =~ /([\d.]+)\s*(min|hours|days)/;
		$file{posted} = (time - $value * $time{$multiplier});
	    }

	    { # Figure out the real size of the file in bytes.
		($file{size}) = $file{size} =~ /^\s*(.*?)\s*$/;
		if (exists $known_size{$file{size}}) {
		    $file{bytesize} = $known_size{$file{size}};
		} else {
		    my ($val, $mult) = $file{size} =~ /([\d.]+)\s*(bytes|K|Megs)/;
		    $file{bytesize} = int($val * $size{$mult});
		}
	    }

	    { # search for the *real* filename in the subject
		(my $re = $filename) =~ s/^_/\[^a-zA-Z0-9\\s\]+/; # Underscore helt frst er litt tricky...
#		$re =~ s/[^\w.-]+/_/g;
		$re =~ s/_/\[\\W_\]+/g;
		$re =~ s/\./\\./;

		if (my ($realname) = $file{subject} =~ /($re)/) {
		    $file{realname} = $realname;
		}
	    }

#	    print Dumper \%file;
	    print STDERR "." if $self->{_debug};
	    push @files, \%file;
	}
	print STDERR " (done)\n" if $self->{_debug};
#	warn "I missed some files\n" unless @files == @results;

	{ # Where to go next...
	    my @links = $center->look_down(_tag => "a", sub {$_[0]->look_down(_tag => "font", size => "-1")});

	    if (@links and $links[-1]->as_text eq '>>') {
		$self->{_next_url} = URI->new_abs($links[-1]->attr("href"), $self->{_next_url})->as_string;
	    } else {
		$self->{_next_url} = $self->_next_group;
	    }
	}
	return \@files;
    }
}

# internal
sub _get_and_parse {
    my $self  = shift;
    my ($url) = @_;
    my $tree  = new HTML::TreeBuilder;
    my $req   = GET $url;
    $req->authorization_basic($self->{_user}, $self->{_pass});

    print STDERR "Fetching page " if $self->{_debug};
    my $res   = $ua->request($req, sub { print STDERR "." if $self->{_debug}; $tree->parse($_[0]) }, 4096);
    $tree->eof;
    print STDERR " (done)\n" if $self->{_debug};

    return ($res, $tree);
}

# internal
sub _get_result_count {
    my $self = shift;
    my ($text) = @_;
    my ($hits) = $text =~ /(\d+)\s+matching\s+file\(s\)/;
    return $hits;
}

sub user {
    my $self = shift;
    $self->{_user} = $_[0] if @_;
    return;
}

sub pass {
    my $self = shift;
    $self->{_pass} = $_[0] if @_;
    return;
}

1;
__END__

=head1 NAME

WWW::Search::Easynews - class for searching for files at Easynews

=head1 SYNOPSIS

  use WWW::Search;
  my $search = new WWW::Search("Easynews");

=head1 DESCRIPTION

Easynews.com is a subscription usenet-service. They have a website with 
archives of the last few days of binary-posts from many groups. 
This module is useless unless you have an account at Easynews.com.

This class is an Easynews specialization of WWW::Search.
It handles making and interpreting Easynews searches

=head1 IMPORTANT

It is a violation of the Easynews TOS to share one account between several people.
So it is important that you make sure that every user of the scripts that use these 
modules log in with their own account. Look in the examples/ directory for examples 
of scripts that does this.

It is also important to remember that you are only allowed to log in to easynews.com 
from two different domains. If you already log in from two domains, and use these 
modules from a third domain, your account will be temporarily closed.

=head1 METHODS

=over 4

=item user($uname)

Sets the username.

=item pass($password)

Sets the password.

=head2 Inherited from WWW::Search

See L<WWW::Search> for more detailed information about these and other inherited functions.

=item new($backend)

To create a new WWW::Search::Easynews object, call

=over 4

  my $search = new WWW::Search('Easynews');

=back

=item native_query($search_string, \%hash_of_options)

This function sets up a query. 

The first argument is a uri-escaped search-string. 
You can use the function WWW::Search::escape_query() for the uri-escaping.

The second is a reference to a hash of options for the search. The hash of options is optional in WWW::Search, 
but to search Easynews, you must enter the name of at least one newsgroup to search, so in this case, the 
options-hash is required.

See C<OPTIONS> for more info about each option.

=head2 Inherited from WWW::SearchResult

See L<WWW::SearchResult> for more information about these and other inherited functions.

=item title()

Easynews substitutes all non-word-characters to _ in filenames. This module looks for 
the original filename in the description of the search-results.
This function returns the original filename if it is found, otherwise it returns the underscore-converted name.

=item description()

Returns the subject used when the file was posted to usenet.

=item url()

Returns the full url to the file. 
This url includes the underscore-converted filename.
To get the url with the original filename, see C<urls()>.

=item urls()

Returns a reference to an array of urls.
The array contains two urls for every file. The first has the underscore-convereted filename (as returned by url()).
The second has the original filename.

=item size()

Returns the approximate size in bytes of the file.

Easynews displayes a short, rounded number for the filesize (like "14.6 Megs"). 
This function calculates the size in bytes before it returns it. Several commonly used filesizes
used in multipart-postings has been hardcoded into the module so that atleast _some_ files show the exact size :)

=item change_date()

Returns an approximation of the date/time when the file was posted.

Easynews reports the age of the file as a rounded number in minutes, hours or days (depending of how old it is)
This function uses this value to determine the epoch-time of when the file was posted.

This is not very accurate, so this function will return different timestamps for the same file between two different searches.

=back

=head1 OPTIONS

The second argument to native_query() is a hash of options. 
This hash can contain the following keys:

=over 4

=item groups

The only required key.

Its value can be a string with the name of a single group to search, 
or a reference to an array of groups to search.

=item reverse

if C<reverse> is set to C<1>, the hits are returned in reverse order. Default is C<0>

=item pageby

sets the number of hits you want Easynews to return for every page it loads. Default is C<100>

=back

=head1 EXPORT

Nothing.

=head1 EXAMPLES

=over 4

  use WWW::Search;
  my $search = new WWW::Search("Easynews");
  $search->user("username");
  $search->pass("password");

  my @groups = qw/alt.binaries.multimedia alt.binaries.movies/;
  $search->native_query($searchstring, { groups => \@groups });

  while (my $res = $search->next_result) {
    printf "%s\n  %s (%d bytes)\n  %s\n\n", 
        $res->description, $res->title, $res->size, $res->url;
  }

=back

look in the examples/ directory for more examples

=head1 AUTHOR

Trond Michelsen <trondmm-perl@crusaders.no>

=head1 SEE ALSO

L<WWW::Search> and L<WWW::SearchResult>

=cut
