package WWW::Search::Easynews::Groups;

require 5.005;
use strict;
use vars qw/@ISA $VERSION/;

local $^W = 1;

use Carp qw(carp croak);
use WWW::Search qw(generic_option);
require WWW::SearchResult;

use LWP::UserAgent;
use HTTP::Request::Common qw/GET/;

@ISA = qw(WWW::Search);
$VERSION = '0.10';


# Preloaded methods go here.

sub native_setup_search {
    my $self = shift;
    my ($native_query, $native_options_ref) = @_;

    $self->user_agent("Mozilla/4.0 (compatible; MSIE 4.01; Windows 98)");
    $self->{_next_to_retrieve} = 0;

    if (!defined($self->{_options})) {
	$self->{_options} = {
	    'search_url' => 'http://members.easynews.com/index.html',
	};
    };
    my($options_ref) = $self->{_options};
    if (defined($native_options_ref)) {
	# Copy in new options.
	foreach (keys %$native_options_ref) {
	    $options_ref->{$_} = $native_options_ref->{$_};
	};
    };

    # A username and password is essential
    croak "Missing username for easynews" unless defined $self->{_user};
    croak "Missing password for easynews" unless defined $self->{_pass};

    # Process the options.
    # (Now in sorted order for consistency regarless of hash ordering.)

    my $options = join "&" => 
	map  {join "=", $_, $options_ref->{$_}} 
        grep {not generic_option($_) } sort keys %$options_ref;
    
    $self->{_debug} = $options_ref->{'search_debug'};
    $self->{_debug} = 2 if ($options_ref->{'search_parse_debug'});
    $self->{_debug} = 0 if (!defined($self->{_debug}));
    
    # Finally figure out the url.
    $self->{_base_url} = 
	$self->{_next_url} =
	    $self->{_options}{'search_url'} .
		"?" . $options . "&" . 
		    "search=" . $native_query;
    print STDERR "Base URL: $self->{_base_url}\n" if $self->{_debug};
}

sub native_retrieve_some {
    my $self = shift;

    return unless defined $self->{_next_url};

    # get some
    print STDERR __PACKAGE__ . "::native_retrieve_some: fetching " . $self->{_next_url} . "\n" if ($self->{_debug});
    my($res, $tree) = $self->http_request(GET => $self->{_next_url});
    $self->{response} = $res;
    unless ($res->is_success) {
	print STDERR $res->status_line if $self->{_debug};
	return undef;
    }
    my $content = $res->content;
    print STDERR "Content: \n\n$content" if $self->{_debug} >= 4;
    my ($pre) = $content =~ m{<pre>(.*)</pre>}gs;
    print STDERR "Parse this: \n\n$pre" if $self->{_debug} >= 3;
    my @entries = split /[\n\r]/, $pre;
    {
	local $, = "\n------\n";
	local $\ = "\n\n";
	print STDERR @entries if $self->{_debug} >= 3;
    }
    $self->approximate_result_count(scalar @entries);

    my $i = 0;
    while (defined (my $entry = shift @entries)) {
	my ($the_one) = split /\s*\|\s*/, $entry;
	print STDERR ++$i, ": $the_one\n\n" if $self->{_debug} >= 2;
	my ($count, $name) = $the_one =~ m{\((\d+)\).*<a[^>]*>(.*)</a>}sg;
	print STDERR "$name ($count)\n" if $self->{_debug};
	my $hit = new WWW::SearchResult;
	$hit->title($name);
	$hit->size($count);
	my $url = URI->new_abs("/group.html?group=$name", $self->{_next_url})->as_string;
	$hit->add_url($url);
	push(@{$self->{cache}}, $hit);
    }
    $self->{_next_url} = undef;
}

sub http_request {
    my $self = shift;
    my ($proto, $url) = @_;
    my $ua = new LWP::UserAgent;
    my $req  = GET $url;
    $req->authorization_basic($self->{_user}, $self->{_pass});

    print STDERR "Fetching page " if $self->{_debug};
    my $res   = $ua->request($req);
    print STDERR " (done)\n" if $self->{_debug};

    return $res;
}

sub user {
    my $self = shift;
    $self->{_user} = $_[0] if @_;
    return;
}

sub pass {
    my $self = shift;
    $self->{_pass} = $_[0] if @_;
    return;
}


1;
__END__
# Below is stub documentation for your module. You better edit it!

=head1 NAME

WWW::Search::Easynews::Groups - class for searching for newsgroups at Easynews

=head1 SYNOPSIS

  use WWW::Search::Easynews::Groups;
  my $search = new WWW::Search("Easynews::Groups");

=head1 DESCRIPTION

Easynews.com is a subscription usenet-service. They have a website with 
archives of the last few days of binary-posts from many groups. 
This module is useless unless you have an account at Easynews.com.

This class is an Easynews specialization of WWW::Search.
It handles making and interpreting Easynews searches

=head1 METHODS

=over 4

=item user($uname)

Sets the username.

=item pass($password)

Sets the password.

=head2 Inherited from WWW::Search

See L<WWW::Search> for more detailed information about these and other inherited funtions.

=item new($backend)

To create a new WWW::Search::Easynews::Groups object, call

    C<my $search = new WWW::Search('Easynews::Groups');>

=item native_query($search_string, \%hash_of_options)

This function sets up a query. 

The first argument is a uri-escaped search-string. 
You can use the function WWW::Search::escape_query() for the uri-escaping.
To search for groups that match either "sounds" or "mp3", the search-string must be "sounds|mp3". 
There is currently no way to search for groups that matches all words.

The second is a reference to a hash of options for the search. The hash of options is optional.

See C<OPTIONS> for info about each option.

=back

=head1 EXPORT

Nothing

=head1 AUTHOR

Trond Michelsen trondmm-perl@crusaders.no

=head1 SEE ALSO

L<WWW::Search> and L<WWW::SearchResult>

=cut
